/**
  *  \file int/context.cc
  *  \brief Interpreter: Contexts
  */

#include "int/context.h"
#include "int/exec.h"
#include "int/intglobal.h"

/* We assume nil is negative. Otherwise, we'd have to convert in lookup(). */
CTASSERT(IntVariableNames::nil < 0);

/**************************** IntSingleContext ***************************/

bool
IntSingleContext::next()
{
    return false;
}

/************************ IntExecutionFrameContext ***********************/

IntExecutionFrameContext::IntExecutionFrameContext(IntExecutionFrame& frame)
    : frame(frame)
{ }

IntExecutionFrameContext::~IntExecutionFrameContext()
{ }

int32_t
IntExecutionFrameContext::lookup(string_t name)
{
    return frame.local_names.lookup(name);
}
void
IntExecutionFrameContext::set(int32_t index, IntValue* name)
{
    frame.local_values.set(index, name);
}

IntValue*
IntExecutionFrameContext::get(int32_t index)
{
    return IntValue::cloneOf(frame.local_values[index]);
}

IntExecutionFrameContext*
IntExecutionFrameContext::clone()
{
    return new IntExecutionFrameContext(frame);
}
