/**
  *  \file int/if/ifutil.cc
  *  \brief Interpreter Game Interface: Utilities
  */

#include "int/if/ifutil.h"
#include "int/interror.h"
#include "int/value.h"

/** Look up a name in a mapping list.
    \param name    Name of property to find
    \param mapping Mapping list, must be sorted
    \param size    Size of mapping list
    \return index such that mapping[return].name == name, or -1 */
int32_t
lookupName(string_t name, const TIntMapping* mapping, uint32_t size)
{
    uint32_t low = 0;
    uint32_t high = size;

    // Binary intersection
    while (high - low > 3) {
        uint32_t mid = (high + low) / 2;
        if (name < mapping[mid].name)
            high = mid;
        else
            low = mid;
    }

    // Linear search
    while (low < high) {
        if (name == mapping[low].name) {
            return low;
        }
        ++low;
    }
    return -1;
}

void
checkArgumentCount(uint32_t have, uint32_t want)
{
    checkArgumentCount(have, want, want);
}

void
checkArgumentCount(uint32_t have, uint32_t min, uint32_t max)
{
    if (have < min)
        throw IntError("Too few arguments");
    if (have > max)
        throw IntError("Too many arguments");
}

bool
checkIntArg(int32_t& out, IntValue* value)
{
    if (value == 0)
        return false;

    IntIntValue* iv = dynamic_cast<IntIntValue*>(value);
    if (!iv)
        throw IntError::typeError(IntError::tyInteger);

    out = iv->getValue();
    return true;
}

bool
checkIntArg(int32_t& out, IntValue* value, int32_t min, int32_t max)
{
    bool result = checkIntArg(out, value);
    if (result) {
        if (out < min || out > max)
            throw IntError::rangeError();
    }
    return result;
}
