/**
  *  \file int/if/initif.cc
  *  \brief Interpreter Game Interface: Initialisation
  */

#include "int/exec.h"
#include "int/if/globalif.h"
#include "int/if/hullif.h"
#include "int/if/initif.h"
#include "int/if/ionif.h"
#include "int/if/mineif.h"
#include "int/if/planetif.h"
#include "int/if/playerif.h"
#include "int/if/shipif.h"
#include "int/if/specif.h"
#include "int/if/ufoif.h"
#include "int/intglobal.h"

void
initInterpreterGameInterface()
{
    // Add global variables
    global_values.setNew(global_property_names.addMaybe("BEAM"),           new IntSimpleIndexableValue(IFBeamGet,           IFBeamMake));
    global_values.setNew(global_property_names.addMaybe("CFG"),            new IntSimpleIndexableValue(IFCfgGet,            0));
    global_values.setNew(global_property_names.addMaybe("DISTANCE"),       new IntSimpleIndexableValue(IFDistanceGet,       0));
    global_values.setNew(global_property_names.addMaybe("ENGINE"),         new IntSimpleIndexableValue(IFEngineGet,         IFEngineMake));
    global_values.setNew(global_property_names.addMaybe("HULL"),           new IntSimpleIndexableValue(IFHullGet,           IFHullMake));
    global_values.setNew(global_property_names.addMaybe("ISSPECIALFCODE"), new IntSimpleIndexableValue(IFIsSpecialFCodeGet, 0));
    global_values.setNew(global_property_names.addMaybe("LAUNCHER"),       new IntSimpleIndexableValue(IFLauncherGet,       IFLauncherMake));
    global_values.setNew(global_property_names.addMaybe("MINEFIELD"),      new IntSimpleIndexableValue(IFMinefieldGet,      IFMinefieldMake));
    global_values.setNew(global_property_names.addMaybe("PLANET"),         new IntSimpleIndexableValue(IFPlanetGet,         IFPlanetMake));
    global_values.setNew(global_property_names.addMaybe("PLANETNAME"),     new IntSimpleIndexableValue(IFPlanetNameGet,     0));
    global_values.setNew(global_property_names.addMaybe("PLAYER"),         new IntSimpleIndexableValue(IFPlayerGet,         IFPlayerMake));
    global_values.setNew(global_property_names.addMaybe("RANDOMFCODE"),    new IntSimpleIndexableValue(IFRandomFCode,       0));
    global_values.setNew(global_property_names.addMaybe("SHIP"),           new IntSimpleIndexableValue(IFShipGet,           IFShipMake));
    global_values.setNew(global_property_names.addMaybe("SHIPNAME"),       new IntSimpleIndexableValue(IFShipNameGet,       0));
    global_values.setNew(global_property_names.addMaybe("STORM"),          new IntSimpleIndexableValue(IFIonGet,            IFIonMake));
    global_values.setNew(global_property_names.addMaybe("TORPEDO"),        new IntSimpleIndexableValue(IFTorpedoGet,        IFTorpedoMake));
    global_values.setNew(global_property_names.addMaybe("TRUEHULL"),       new IntSimpleIndexableValue(IFTruehull,          0));
    global_values.setNew(global_property_names.addMaybe("UFO"),            new IntSimpleIndexableValue(IFUfoGet,            IFUfoMake));

    // Add global context (=properties)
    IntExecutionContext::setNewGlobalContext(new IntGlobalContext());
}
