/**
  *  \file int/if/planetif.h
  *  \brief Interpreter Game Interface: Planets
  */
#ifndef PCC_V2_INT_IF_PLANETIF_H
#define PCC_V2_INT_IF_PLANETIF_H

#include "cpluslib/types.h"
#include "int/context.h"
#include "cpluslib/string.h"

class IntValue;
class IntDataSegment;
class GPlanet;

IntValue*   IFPlanetGet(IntDataSegment& dseg, uint32_t index, uint32_t nargs);
IntContext* IFPlanetMake();

/** Ship context. */
class IntPlanetContext : public IntContext {
    int pid;
 public:
    IntPlanetContext(int pid);
    ~IntPlanetContext();

    // IntContext:
    int32_t   lookup(string_t name);
    void      set(int32_t index, IntValue* value);
    IntValue* get(int32_t index);
    bool      next();
    IntContext* clone();
};

enum IntPlanetProperty {
    ippBaseDefenseWanted,
    ippBaseFlag,
    ippBaseStr,
    ippColonistChange,
    ippColonistChangeStr,
    ippColonistHappy,
    ippColonistHappyStr,
    ippColonistSupported,
    ippColonistTax,
    ippColonists,
    ippDefense,
    ippDefenseMax,
    ippDefenseWanted,
    ippDensityD,
    ippDensityM,
    ippDensityN,
    ippDensityT,
    ippFCode,
    ippFactories,
    ippFactoriesMax,
    ippFactoriesWanted,
    ippGroundD,
    ippGroundM,
    ippGroundN,
    ippGroundT,
    ippId,
    ippIndustry,
    ippIndustryCode,
    ippLevel,
    ippLocX,
    ippLocY,
    ippMarked,
    ippMinedD,
    ippMinedM,
    ippMinedN,
    ippMinedStr,
    ippMinedT,
    ippMines,
    ippMinesMax,
    ippMinesWanted,
    ippMoney,
    ippName,
    ippNativeChange,
    ippNativeChangeStr,
    ippNativeGov,
    ippNativeGovCode,
    ippNativeHappy,
    ippNativeHappyStr,
    ippNativeRace,
    ippNativeRaceCode,
    ippNativeTax,
    ippNatives,
    ippOrbitingEnemies,
    ippOrbitingOwn,
    ippOrbitingShips,
    ippSupplies,
    ippTask,
    ippTaskBase,
    ippTemp,
    ippTempStr
};

IntValue* getPlanetProperty(const GPlanet& pl, IntPlanetProperty ipp);

#endif
