/**
  *  \file int/parseexpr.h
  *  \brief Interpreter: Expression Parser
  */
#ifndef PCC_V2_INT_PARSEEXPR_H
#define PCC_V2_INT_PARSEEXPR_H

#include "cpluslib/ptrvector.h"
#include "int/expr.h"
#include "int/lex.h"

/** Expression Parser. This parses an expression, tokenized by an IntTokenizer,
    into a tree of IntExprNode's.

    To support exception safety, this collects expression trees on a stack which
    is cleaned up in the destructor. During parsing, several partial trees exist
    on the stack. When parsing finishes, only one tree (the result) remains, which
    refers to all former partial trees. */
class IntExpressionParser {
 public:
    IntExpressionParser(IntTokenizer& tok);

    IntExprNode* parse();

 private:
    void parseSequence();
    void parseAssignment();
    void parseOr();
    void parseAnd();
    void parseNot();
    void parseComparison();
    void parseConcat();
    void parseAdd();
    void parseMult();
    void parseNeg();
    void parsePow();
    void parsePrimary();
    void parseArglist(IntFunctionCallNode* fcn);

    void makeUnary(IntSimpleRValueExprNode* n);
    void makeBinary(IntSimpleRValueExprNode* n);
    void makeTernary(IntSimpleRValueExprNode* n);

    IntTokenizer& tok;
    ptr_vector<IntExprNode> stack;
};

#endif
