/**
 *  \file testaa.cc
 *  \brief Bresenham AA line drawing
 *  \author Stefan Reuther
 */

#include <algorithm>                  // std::swap
#include <math.h>                     // sin, cos for test
#include "gfx/init.h"
#include "gfx/gfx.h"
#include "gfx/palette.h"
#include "gfx/gfxaa.h"

// /** Circle drawing, with anti-aliasing.  */
// void
// drawFilledCircleAA(GfxCanvas& can, GfxParameters& parm, int x0, int y0, int r)
// {
//     register int w = 0;
//     register int k = 1;
//     register int z = 0;
//     register int y2 = 0;
//     register int xk = 2*r-1;
//     register int x = r;

//     while (x >= 0) {
//         while (z < y2) {
//             z += k;
//             ++w;
//             k += 2;
//         }
//         if (w >= x) {
//             uint8 alpha = (z - y2) * 255 / k;

//             can.drawPixel(x0 - x, y0 - w, parm.color, ~alpha);
// //             can.drawPixel(x0 - x, y0 - w+1, parm.color, alpha);

//              can.drawPixel(x0 + x, y0 - w, parm.color, ~alpha);
// //             can.drawPixel(x0 + x, y0 - w+1, parm.color, alpha);

//              can.drawPixel(x0 - w, y0 - x, parm.color, ~alpha);
//              can.drawPixel(x0 + w, y0 - x, parm.color, ~alpha);
//              can.drawHLine(x0 - w+1, y0 - x, x0 + w, parm.color, 255, 255);
            
//              can.drawPixel(x0 - w, y0 + x, parm.color, ~alpha);
//              can.drawPixel(x0 + w, y0 + x, parm.color, ~alpha);
//              can.drawHLine(x0 - w+1, y0 + x, x0 + w, parm.color, 255, 255);

//              can.drawPixel(x0 - x, y0 + w, parm.color, ~alpha);
// //             can.drawPixel(x0 - x, y0 + w-1, parm.color, alpha);
            
//              can.drawPixel(x0 + x, y0 + w, parm.color, ~alpha);
// //             can.drawPixel(x0 + x, y0 + w-1, parm.color, alpha);
//         }
//         y2 += xk;
//         xk -= 2;
//         --x;
//     }
// }


static void
test(GfxCanvas& screen, GfxParameters& parm, int x1, int y1, int x2, int y2)
{
    drawLineAA(screen, parm, GfxPoint(x1, y1), GfxPoint(x2, y2));
    x1 += 320;
    x2 += 320;
    drawLine(screen, parm, GfxPoint(x1, y1), GfxPoint(x2, y2));
}

static void
testC(GfxCanvas& screen, GfxParameters& parm, int x1, int y1, int r)
{
    drawCircleAA(screen, parm, GfxPoint(x1, y1), r);
    x1 += 320;
    drawCircle(screen, parm, GfxPoint(x1, y1), r);
}

// static void
// testF(GfxCanvas& screen, GfxParameters& parm, int x1, int y1, int r)
// {
//     drawFilledCircleAA(screen, parm, x1, y1, r);
//     x1 += 320;
//     drawFilledCircle(screen, parm, x1, y1, r);
// }

int main()
{
    SDL_Surface* sfc = initGraphics(0,0);
    initPalette(sfc);
    GfxSurface screen(sfc, true);

    GfxParameters parm;
    parm.color = standard_colors[COLOR_RED];
    screen.lock();

    drawSolidBar(screen, GfxRect(screen), standard_colors[COLOR_WHITE]);

    for (int i = 0; i < 10; ++i) {
        drawSolidBar(screen, GfxRect(20*i, 0, 10, 100), standard_colors[COLOR_YELLOW]);
        drawSolidBar(screen, GfxRect(320+20*i, 0, 10, 100), standard_colors[COLOR_YELLOW]);
    }

#if 0
    /* a single line */
    test(screen, parm, 100, 1, 102, 40);
    test(screen, parm, 102, 41, 104, 80);
#endif
#if 0
    /* some random lines */
    for (int i = 0; i < 100; ++i) {
        int x1 = rand() % 300, y1 = rand() % 400, x2 = rand() % 300, y2 = rand() % 400;
        test(screen, parm, x1, y1, x2, y2);
    }
#endif
#if 0
    /* a moire circle */
    for (int i = 0; i < 360; ++i) {
        test(screen, parm, 150, 150, 150+int(150*sin(3.141595/180*i)+.5), 150+int(150*cos(3.141595/180*i)+.5));
    }
#endif
#if 0
    /* a circle outline (not a good algorithm, as many points are drawn twice) */
    for (int i = 0; i < 360; ++i) {
        test(screen, parm,
             150+int(150*sin(3.141595/180*i)+.5), 150+int(150*cos(3.141595/180*i)+.5),
             150+int(150*sin(3.141595/180*(i+1))+.5), 150+int(150*cos(3.141595/180*(i+1))+.5));
    }
#endif
#if 0
    /* some simple circles */
    testC(screen, parm, 100, 100, 50);
    testC(screen, parm, 8, 10, 5);
    testC(screen, parm, 20, 10, 4);
    testC(screen, parm, 30, 10, 3);
#endif
#if 1
    /* random circles */
    for (int i = 0; i < 100; ++i)
        testC(screen, parm, rand() % 100 + 100, rand() % 300, rand() % 80 + 20);
#endif
#if 0
    /* concentric circles without gaps; gives nice moire patterns in both cases */
    for (int i = 0; i < 100; ++i)
        testC(screen, parm, 105, 105, i);
#endif
#if 0
    for (int i = 0; i < 100; ++i)
        //testC(screen, parm, 10+i, 10+i, i);
        testC(screen, parm, 160-i/3, 160-i/3, i);
#endif
#if 0
    /* concentric circles with gap, gives nice circles with AA, moire without */
    for (int i = 0; i < 100; i += 2)
        testC(screen, parm, 105, 105, i);
#endif
// #if 1
//     /* some simple filled circles */
//     testF(screen, parm, 100, 100, 50);
//     testF(screen, parm, 8, 10, 5);
//     testF(screen, parm, 20, 10, 4);
//     testF(screen, parm, 30, 10, 3);
// #endif
    screen.unlock();
    screen.update(GfxRect(screen));

    getchar();
}
